@@ -18,9 +18,9 @@
 
 
 
-static GdkPixbuf *button_load_pixbuf (const char  *file,
-				      char       **error);
-static GdkPixbuf * get_missing (int preffered_size);
+static GdkPixbuf * get_missing (GtkIconTheme *icon_theme,
+				int           preffered_size);
+static void button_widget_icon_theme_changed (ButtonWidget *button);
 
 enum {
 	PROP_0,
@@ -28,8 +28,7 @@ enum {
 	PROP_HAS_ARROW,
 	PROP_DND_HIGHLIGHT,
 	PROP_ORIENTATION,
-	PROP_ICON_NAME,
-	PROP_STOCK_ID
+	PROP_ICON_NAME
 };
 
 #define BUTTON_WIDGET_DISPLACEMENT 2
@@ -132,6 +131,13 @@ button_widget_realize(GtkWidget *widget)
 	gdk_window_set_user_data (button->event_window, widget);
 
 	widget->style = gtk_style_attach (widget->style, widget->window);
+
+	BUTTON_WIDGET (widget)->icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
+	g_signal_connect_object (BUTTON_WIDGET (widget)->icon_theme, "changed",
+				 G_CALLBACK (button_widget_icon_theme_changed),
+				 button,
+				 G_CONNECT_SWAPPED);
+
 }
 
 static void
@@ -164,97 +170,47 @@ button_widget_unset_pixbufs (ButtonWidget *button)
 		g_object_unref (button->pixbuf);
 	button->pixbuf = NULL;
 
-	if (button->scaled)
-		g_object_unref (button->scaled);
-	button->scaled = NULL;
-
-	if (button->scaled_hc)
-		g_object_unref (button->scaled_hc);
-	button->scaled_hc = NULL;
-}
-
-static void
-button_widget_load_pixbuf_and_scale (ButtonWidget *button)
-{
-	double scale;
-	int    width;
-	int    height;
-
-	if (button->size <= 1)
-		return;
-
-	if (button->pixbuf == NULL) {
-		if (!button->filename && !button->stock_id)
-			return;
-
-		if (button->filename != NULL) {
-			char *error = NULL;
-
-			button->pixbuf = button_load_pixbuf (button->filename, &error);
-			if (error) {
-				panel_error_dialog (gdk_screen_get_default (),
-						    "cannot_load_pixbuf",
-						    _("Failed to load image %s"),
-						    "%s",
-						    button->filename,
-						    error);
-				g_free (error);
-			}
-		}
-
-		if (button->pixbuf == NULL &&
-		    button->stock_id != NULL)
-			button->pixbuf = gtk_widget_render_icon (
-						GTK_WIDGET (button),
-						button->stock_id,
-						(GtkIconSize) -1,
-						NULL);
-
-/* FIXME: this should be based on the panel size */
-#define PREFERRED_SIZE 48
-		if (button->pixbuf == NULL)
-			button->pixbuf = get_missing (PREFERRED_SIZE);
-#undef PREFERRED_SIZE
-
-		if (!button->pixbuf)
-			return;
-	}
-
-	width  = gdk_pixbuf_get_width  (button->pixbuf);
-	height = gdk_pixbuf_get_height (button->pixbuf);
-
-	if (button->orientation & PANEL_HORIZONTAL_MASK)
-		scale = (double) button->size / height;
-	else
-		scale = (double) button->size / width;
-
-	width  *= scale;
-	height *= scale;
-
-	if (button->scaled) {
-		if (gdk_pixbuf_get_width  (button->scaled) == width &&
-		    gdk_pixbuf_get_height (button->scaled) == height)
-			return; /* no need to re-scale */
-
-		g_object_unref (button->scaled);
-	}
-
-	button->scaled = gdk_pixbuf_scale_simple (
-				button->pixbuf, width, height, GDK_INTERP_BILINEAR);
-
-	if (button->scaled_hc)
-		g_object_unref (button->scaled_hc);
-	
-	button->scaled_hc = make_hc_pixbuf (button->scaled);
-
-	gtk_widget_queue_resize (GTK_WIDGET (button));
+	if (button->pixbuf_hc)
+		g_object_unref (button->pixbuf_hc);
+	button->pixbuf_hc = NULL;
 }
 
 static void
 button_widget_reload_pixbuf (ButtonWidget *button)
 {
 	button_widget_unset_pixbufs (button);
-	button_widget_load_pixbuf_and_scale (button);
+
+	if (button->size <= 1)
+		return;
+
+	if (button->filename != NULL) {
+		char *error = NULL;
+
+		button->pixbuf = panel_load_icon (button->icon_theme,
+						  button->filename,
+						  button->size,
+						  &error);
+		if (error) {
+			panel_error_dialog (gdk_screen_get_default (),
+					    "cannot_load_pixbuf", TRUE,
+					    _("Failed to load image %s"),
+					    "%s",
+					    button->filename,
+					    error);
+			g_free (error);
+		}
+	}
+
+	if (button->pixbuf == NULL)
+		button->pixbuf = get_missing (button->icon_theme,
+					      button->size);
+
+	if (button->pixbuf == NULL)
+		return;
+
+	button->pixbuf_hc = make_hc_pixbuf (button->pixbuf);
+
+	gtk_widget_queue_resize (GTK_WIDGET (button));
 }
 
 static void
@@ -264,13 +220,6 @@ button_widget_icon_theme_changed (ButtonWidget *button)
 		button_widget_reload_pixbuf (button);
 }
 
-static void
-button_widget_gtk_theme_changed (ButtonWidget *button)
-{
-	if (button->stock_id != NULL)
-		button_widget_reload_pixbuf (button);
-}
-
 static void
 button_widget_finalize (GObject *object)
 {
@@ -281,9 +230,6 @@ button_widget_finalize (GObject *object)
 	g_free (button->filename);
 	button->filename = NULL;
 
-	g_free (button->stock_id);
-	button->stock_id = NULL;
-	
 	parent_class->finalize (object);
 }
 
@@ -315,9 +261,6 @@ button_widget_get_property (GObject    *object,
 	case PROP_ICON_NAME:
 		g_value_set_string (value, button->filename);
 		break;
-	case PROP_STOCK_ID:
-		g_value_set_string (value, button->stock_id);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -352,100 +295,26 @@ button_widget_set_property (GObject      *object,
 	case PROP_ICON_NAME:
 		button_widget_set_icon_name (button, g_value_get_string (value));
 		break;
-	case PROP_STOCK_ID:
-		button_widget_set_stock_id (button, g_value_get_string (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-static char *default_pixmap = NULL;
-
 static GdkPixbuf *
-get_missing (int preffered_size)
+get_missing (GtkIconTheme *icon_theme,
+	     int           preffered_size)
 {
 	GdkPixbuf *retval = NULL;
 
-	if (default_pixmap == NULL)
-		default_pixmap = gnome_desktop_item_find_icon (panel_icon_theme, "gnome-unknown.png",
-							       preffered_size, 0);
-	if (default_pixmap != NULL)
-		retval = gdk_pixbuf_new_from_file_at_size (default_pixmap,
-							   preffered_size,
-							   preffered_size,
-							   NULL);
+	retval = gtk_icon_theme_load_icon (icon_theme, "gnome-unknown",
+					   preffered_size, 0, NULL);
 	if (retval == NULL)
 		retval = missing_pixbuf (preffered_size);
 
 	return retval;
 }
 
-static GdkPixbuf *
-load_pixbuf (const char  *file,
-	     int          preffered_size,
-	     char       **str_error)
-{
-	GError    *error = NULL;
-	GdkPixbuf *retval = NULL;
-	char      *full;
-
-	if (g_path_is_absolute (file))
-		full = g_strdup (file);
-	else
-		full = gnome_desktop_item_find_icon (panel_icon_theme, file, preffered_size, 0);
-
-	if (full) {
-		retval = gdk_pixbuf_new_from_file_at_size (full,
-							   preffered_size,
-							   preffered_size,
-							   &error);
-		if (!retval) {
-			if (str_error)
-				*str_error = g_strdup (error ? error->message : _("none"));
-			g_error_free (error);
-		}
-		g_free (full);
-	} else if (str_error)
-		*str_error = g_strdup (_("file not found"));
-
-	return retval;
-
-
-}
-
-static GdkPixbuf *
-button_load_pixbuf (const char  *file,
-		    char       **error)
-{
-/* FIXME: this should be based on the panel size */
-#define PREFERRED_SIZE 48
-
-	GdkPixbuf *retval = NULL;
-
-	if (string_empty (file))
-		return NULL;
-
-	retval = load_pixbuf (file, PREFERRED_SIZE, NULL);
-	if (!retval) {
-		char *tmp;
-
-		if (error && *error) {
-			g_free (*error);
-			*error = NULL;
-		}
-
-		tmp = g_path_get_basename (file);
-		retval = load_pixbuf (tmp, PREFERRED_SIZE, error);
-		g_free (tmp);
-	}
-
-	return retval;
-
-#undef PREFERRED_SIZE
-}
-
 static GtkArrowType
 calc_arrow (PanelOrientation  orientation,
 	    int               button_width,
@@ -509,7 +378,7 @@ button_widget_expose (GtkWidget         *widget,
 	if (!GTK_WIDGET_VISIBLE (widget) || !GTK_WIDGET_MAPPED (widget))
 		return FALSE;
 
-	if (!button_widget->scaled_hc && !button_widget->scaled)
+	if (!button_widget->pixbuf_hc && !button_widget->pixbuf)
 		return FALSE;
 
 	/* offset for pressed buttons */
@@ -517,16 +386,16 @@ button_widget_expose (GtkWidget         *widget,
 		BUTTON_WIDGET_DISPLACEMENT * widget->allocation.height / 48.0 : 0;
 
 	if (!button_widget->activatable) {
-		pb = gdk_pixbuf_copy (button_widget->scaled);
-		gdk_pixbuf_saturate_and_pixelate (button_widget->scaled,
+		pb = gdk_pixbuf_copy (button_widget->pixbuf);
+		gdk_pixbuf_saturate_and_pixelate (button_widget->pixbuf,
 						  pb,
 						  0.8,
 						  TRUE);
 	} else if (panel_global_config_get_highlight_when_over () && 
 	    (button->in_button || GTK_WIDGET_HAS_FOCUS (widget)))
-		pb = g_object_ref (button_widget->scaled_hc);
+		pb = g_object_ref (button_widget->pixbuf_hc);
 	else
-		pb = g_object_ref (button_widget->scaled);
+		pb = g_object_ref (button_widget->pixbuf);
 
 	g_assert (pb != NULL);
 
@@ -616,9 +485,9 @@ button_widget_size_request (GtkWidget      *widget,
 {
 	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
 
-	if (button_widget->scaled) {
-		requisition->width  = gdk_pixbuf_get_width  (button_widget->scaled);
-		requisition->height = gdk_pixbuf_get_height (button_widget->scaled);
+	if (button_widget->pixbuf) {
+		requisition->width  = gdk_pixbuf_get_width  (button_widget->pixbuf);
+		requisition->height = gdk_pixbuf_get_height (button_widget->pixbuf);
 	}
 }
 
@@ -638,7 +507,7 @@ button_widget_size_allocate (GtkWidget     *widget,
 	if (button_widget->size != size) {
 		button_widget->size = size;
 
-		button_widget_load_pixbuf_and_scale (button_widget);
+		button_widget_reload_pixbuf (button_widget);
 	}
 
 	widget->allocation = *allocation;
@@ -716,8 +585,7 @@ static void
 button_widget_instance_init (ButtonWidget *button)
 {
 	button->pixbuf    = NULL;
-	button->scaled    = NULL;
-	button->scaled_hc = NULL;
+	button->pixbuf_hc = NULL;
 	
 	button->arrow       = 0;
 	button->orientation = PANEL_ORIENTATION_TOP;
@@ -726,14 +594,6 @@ button_widget_instance_init (ButtonWidget *button)
 	button->dnd_highlight = FALSE;
 
 	button->size = 0;
-
-	g_signal_connect_object (panel_icon_theme, "changed",
-				 G_CALLBACK (button_widget_icon_theme_changed),
-				 button,
-				 G_CONNECT_SWAPPED);
-
-	g_signal_connect (button, "style-set", 
-			  G_CALLBACK (button_widget_gtk_theme_changed), button);
 }
 
 static void
@@ -805,15 +665,6 @@ button_widget_class_init (ButtonWidgetClass *klass)
 					     "The desired icon for the ButtonWidget",
 					     NULL,
 					     G_PARAM_READWRITE));
-
-	g_object_class_install_property (
-			gobject_class,
-			PROP_STOCK_ID,
-			g_param_spec_string ("stock-id",
-					     "Stock Icon ID",
-					     "The desired stock icon for the ButtonWidget",
-					     NULL,
-					     G_PARAM_READWRITE));
 }
 
 GType
@@ -858,23 +709,6 @@ button_widget_new (const char       *filename,
 	return retval;
 }
 
-GtkWidget *
-button_widget_new_from_stock (const char       *stock_id,
-			      gboolean          arrow,
-			      PanelOrientation  orientation)
-{
-	GtkWidget *retval;
-
-	retval = g_object_new (
-			BUTTON_TYPE_WIDGET,
-			"has-arrow", arrow,
-			"orientation", orientation,
-			"stock-id", stock_id,
-			NULL);
-	
-	return retval;
-}
-
 void
 button_widget_set_activatable (ButtonWidget *button,
 			       gboolean      activatable)
@@ -927,32 +761,6 @@ button_widget_get_icon_name (ButtonWidget *button)
 	return button->filename;
 }
 
-void
-button_widget_set_stock_id (ButtonWidget *button,
-			    const char   *stock_id)
-{
-	g_return_if_fail (BUTTON_IS_WIDGET (button));
-	
-	if (button->stock_id && stock_id && !strcmp (button->stock_id, stock_id))
-		return;
-
-	if (button->stock_id)
-		g_free (button->stock_id);
-	button->stock_id = g_strdup (stock_id);
-
-	button_widget_reload_pixbuf (button);
-
-	g_object_notify (G_OBJECT (button), "stock-id");
-}
-
-const char *
-button_widget_get_stock_id (ButtonWidget *button)
-{
-	g_return_val_if_fail (BUTTON_IS_WIDGET (button), NULL);
-	
-	return button->stock_id;
-}
-
 void
 button_widget_set_orientation (ButtonWidget     *button,
 			       PanelOrientation  orientation)
