@@ -20,6 +20,7 @@
 
 #include "eggaccelerators.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
@@ -148,7 +149,7 @@ is_meta (const gchar *string)
 	  (string[2] == 'e' || string[2] == 'E') &&
 	  (string[3] == 't' || string[3] == 'T') &&
 	  (string[4] == 'a' || string[4] == 'A') &&
-	  (string[8] == '>'));
+	  (string[5] == '>'));
 }
 
 static inline gboolean
@@ -160,7 +161,7 @@ is_super (const gchar *string)
 	  (string[3] == 'p' || string[3] == 'P') &&
 	  (string[4] == 'e' || string[4] == 'E') &&
 	  (string[5] == 'r' || string[5] == 'R') &&
-	  (string[8] == '>'));
+	  (string[6] == '>'));
 }
 
 static inline gboolean
@@ -172,7 +173,14 @@ is_hyper (const gchar *string)
 	  (string[3] == 'p' || string[3] == 'P') &&
 	  (string[4] == 'e' || string[4] == 'E') &&
 	  (string[5] == 'r' || string[5] == 'R') &&
-	  (string[8] == '>'));
+	  (string[6] == '>'));
+}
+
+static inline gboolean
+is_keycode (const gchar *string)
+{
+  return ((string[0] == '0') &&
+	  (string[1] == 'x'));
 }
 
 /**
@@ -203,21 +211,26 @@ is_hyper (const gchar *string)
 gboolean
 egg_accelerator_parse_virtual (const gchar            *accelerator,
                                guint                  *accelerator_key,
+			       guint                  *keycode,
                                EggVirtualModifierType *accelerator_mods)
 {
   guint keyval;
   GdkModifierType mods;
   gint len;
   gboolean bad_keyval;
+  gint tmp_keycode;
   
   if (accelerator_key)
     *accelerator_key = 0;
   if (accelerator_mods)
     *accelerator_mods = 0;
+  if (keycode)
+    *keycode = 0;
 
   g_return_val_if_fail (accelerator != NULL, FALSE);
 
   bad_keyval = FALSE;
+  tmp_keycode = 0;
   
   keyval = 0;
   mods = 0;
@@ -314,12 +327,35 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
       else
 	{
           keyval = gdk_keyval_from_name (accelerator);
-          
+
           if (keyval == 0)
-            bad_keyval = TRUE;
-          
+	    {
+	      /* If keyval is 0, than maybe it's a keycode.  Check for 0x## */
+	      if (len >= 4 && is_keycode (accelerator))
+		{
+		  char keystring[5];
+		  gchar *endptr;
+
+		  memcpy (keystring, accelerator, 4);
+		  keystring [4] = '\000';
+
+		  tmp_keycode = strtol (keystring, &endptr, 16);
+
+		  if (endptr == NULL || *endptr != '\000')
+		    {
+		      bad_keyval = TRUE;
+		    }
+		  else
+		    {
+		      /* 0x00 is an invalid keycode too. */
+		      if (tmp_keycode == 0)
+			bad_keyval = TRUE;
+		    }
+		}
+	    }
+
           accelerator += len;
-          len -= len;              
+          len -= len;
 	}
     }
   
@@ -327,6 +363,8 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
     *accelerator_key = gdk_keyval_to_lower (keyval);
   if (accelerator_mods)
     *accelerator_mods = mods;
+  if (keycode)
+    *keycode = tmp_keycode;
 
   return !bad_keyval;
 }
@@ -347,6 +385,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
  */
 gchar*
 egg_virtual_accelerator_name (guint                  accelerator_key,
+			      guint		     keycode,
                               EggVirtualModifierType accelerator_mods)
 {
   static const gchar text_release[] = "<Release>";
@@ -366,9 +405,16 @@ egg_virtual_accelerator_name (guint                  accelerator_key,
 
   accelerator_mods &= EGG_VIRTUAL_MODIFIER_MASK;
 
-  keyval_name = gdk_keyval_name (gdk_keyval_to_lower (accelerator_key));
-  if (!keyval_name)
-    keyval_name = "";
+  if (!accelerator_key)
+    {
+      keyval_name = g_strdup_printf ("0x%02x", keycode);
+    }
+  else
+    {
+      keyval_name = gdk_keyval_name (gdk_keyval_to_lower (accelerator_key));
+      if (!keyval_name)
+        keyval_name = "";
+    }
 
   l = 0;
   if (accelerator_mods & EGG_VIRTUAL_RELEASE_MASK)
