@@ -38,6 +38,7 @@
 #include "panel-globals.h"
 #include "panel-multiscreen.h"
 #include "panel-lockdown.h"
+#include "panel-compatibility.h"
 
 #undef LAUNCHER_DEBUG
 
@@ -533,32 +534,22 @@ create_launcher (const char *location)
 }
 
 static char *
-guess_icon_from_exec (GnomeDesktopItem *ditem)
+guess_icon_from_exec (GtkIconTheme     *icon_theme,
+		      GnomeDesktopItem *ditem)
 {
 	const char *exec;
 	char       *icon_name;
-	char       *icon;
+	char       *retval;
 
 	exec = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC);
 	if (!exec || !exec [0])
 		return NULL;
 
 	icon_name = g_path_get_basename (exec);
+	retval = panel_find_icon (icon_theme, icon_name, 48);
+	g_free (icon_name);
 
-	if (!(icon = gnome_desktop_item_find_icon (panel_icon_theme, icon_name, 48, 0))) {
-		g_free (icon_name);
-		return NULL;
-	}
-
-	if (g_file_test (icon, G_FILE_TEST_IS_DIR)) {
-		g_free (icon_name);
-		g_free (icon);
-		return NULL;
-	}
-
-	g_free (icon);
-
-	return icon_name;
+	return retval;
 }
 
 static void
@@ -598,7 +589,8 @@ setup_button (Launcher *launcher)
 	icon = gnome_desktop_item_get_string (launcher->ditem,
 					      GNOME_DESKTOP_ITEM_ICON);
 	if (!icon)
-		icon = freeme = guess_icon_from_exec (launcher->ditem);
+		icon = freeme = guess_icon_from_exec (BUTTON_WIDGET (launcher->button)->icon_theme,
+						      launcher->ditem);
 
 	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), sure_string (icon));
 
@@ -919,6 +911,7 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	client  = panel_gconf_get_client ();
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "launcher_location");
+	panel_compatibility_migrate_applications_scheme (client, key);
 	launcher_location = gconf_client_get_string (client, key, NULL);
 
 	if (!launcher_location) {
