@@ -3,8 +3,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-ditem-edit.h>
 
 #include "menu-ditem.h"
@@ -23,7 +22,7 @@ ditem_properties_clicked (GtkWidget *w, int response, gpointer data)
 	if (response == GTK_RESPONSE_HELP) {
 		panel_show_help (
 			gtk_window_get_screen (GTK_WINDOW (w)),
-			"user-guide.xml", "gospanel-16");
+			"user-guide.xml", "gospanel-52");
 	} else if (response == REVERT_BUTTON) {
 		if (ditem != NULL)
 			gnome_ditem_edit_set_ditem (dee, ditem);
@@ -123,7 +122,7 @@ ditem_properties_close (GtkWidget *dialog,
 
 	if (saving_error)
 		panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
-				    "cannot_save_entry",
+				    "cannot_save_entry", TRUE,
 				    _("Cannot save changes to launcher"),
 				    "%s",
 				    saving_error);
@@ -444,7 +443,14 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	char *name, *loc, *dir;
 	GtkWidget *dialog; 
 
-	if (response != GTK_RESPONSE_OK) {
+	switch (response) {
+	case GTK_RESPONSE_OK:
+		break;
+	case GTK_RESPONSE_HELP:
+		panel_show_help (gtk_window_get_screen (GTK_WINDOW (d)),
+				 "wgospanel.xml", "gospanel-52");
+		return;
+	default:
 		gtk_widget_destroy (d);
 		return;
 	}
@@ -459,7 +465,7 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	/* check for valid name */
 	if (string_empty (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME))) {
 		dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (d)),
-					     "cannot_create_launcher",
+					     "cannot_create_launcher", TRUE,
 					     _("Cannot create launcher"),
 					     _("You have to specify a name."));
 
@@ -476,7 +482,7 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	    (gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_LINK &&
 	     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL)))) {
 		dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (d)),
-					     "cannot_create_launcher",
+					     "cannot_create_launcher", TRUE,
 					     _("Cannot create launcher"),
 					     _("You have to specify a valid URL or command."));
 		g_signal_connect_swapped (G_OBJECT (dialog),
@@ -507,7 +513,7 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 				 &error);
 	if (error) {
 		panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (d)),
-				    "cannot_save_menu_item" /* class */,
+				    "cannot_save_menu_item" /* class */, TRUE,
 				    _("Cannot save menu item to disk"),
 				    "%s",
 				    error->message);
@@ -532,7 +538,7 @@ panel_new_launcher (const char *item_loc,
 
 	if (!is_item_writable (item_loc, NULL)) {
 		dialog = panel_error_dialog (screen,
-					     "cannot_create_launcher",
+					     "cannot_create_launcher", TRUE,
 					     _("Cannot create launcher"),
 					     _("You can not create a new launcher at this "
 					       "location since the location is not writable."));
@@ -543,6 +549,7 @@ panel_new_launcher (const char *item_loc,
 	dialog = gtk_dialog_new_with_buttons (
 				_("Create Launcher"),
 				NULL, 0 /* flags */,
+				GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				NULL);
