@@ -228,6 +228,15 @@ panel_menu_bar_append_from_desktop (GtkWidget *menu,
 
 	g_key_file_free (key_file);
 
+	if (icon)
+		g_free (icon);
+
+	if (name)
+		g_free (name);
+
+	if (comment)
+		g_free (comment);
+
 	if (path_freeme)
 		g_free (path_freeme);
 }
@@ -298,6 +307,34 @@ panel_menu_bar_append_place_item (PanelMenuBar *menubar,
 	setup_uri_drag (item, uri, icon_name);
 }
 
+static void
+panel_menu_bar_recreate_menus (PanelMenuBar *menubar)
+{
+	if (menubar->priv->applications_menu) {
+		gtk_widget_destroy (menubar->priv->applications_menu);
+		menubar->priv->applications_menu = panel_menu_bar_create_applications_menu (menubar);
+		panel_menu_bar_connect_menu_signals (menubar,
+						     &menubar->priv->applications_menu);
+		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
+					       "menu_panel",
+					       menubar->priv->panel);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->applications_item),
+					   menubar->priv->applications_menu);
+	}
+
+	if (menubar->priv->desktop_menu) {
+		gtk_widget_destroy (menubar->priv->desktop_menu);
+		menubar->priv->desktop_menu = panel_menu_bar_create_desktop_menu (menubar);
+		panel_menu_bar_connect_menu_signals (menubar,
+						     &menubar->priv->desktop_menu);
+		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->desktop_menu),
+					       "menu_panel",
+					       menubar->priv->panel);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->desktop_item),
+					   menubar->priv->desktop_menu);
+	}
+}
+
 static void
 panel_menu_bar_recreate_places_menu (PanelMenuBar *menubar)
 {
@@ -314,22 +351,6 @@ panel_menu_bar_recreate_places_menu (PanelMenuBar *menubar)
 	}
 }
 
-static void
-panel_menu_bar_recreate_desktop_menu (PanelMenuBar *menubar)
-{
-	if (menubar->priv->desktop_menu) {
-		gtk_widget_destroy (menubar->priv->desktop_menu);
-		menubar->priv->desktop_menu = panel_menu_bar_create_desktop_menu (menubar);
-		panel_menu_bar_connect_menu_signals (menubar,
-						     &menubar->priv->desktop_menu);
-		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->desktop_menu),
-					       "menu_panel",
-					       menubar->priv->panel);
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->desktop_item),
-					   menubar->priv->desktop_menu);
-	}
-}
-
 static void
 panel_menu_bar_append_applications_menu (GtkWidget    *menu,
 					 PanelMenuBar *menubar)
@@ -406,9 +427,6 @@ panel_menu_bar_append_gtk_bookmarks (PanelMenuBar *menubar,
 	int          i;
 	GSList      *add_files, *l;
 	GnomeVFSURI *uri;
-	char        *basename;
-	char        *display_uri;
-	char        *tooltip;
 
 	filename = g_build_filename (g_get_home_dir (),
 				     BOOKMARKS_FILENAME, NULL);
@@ -468,10 +486,14 @@ panel_menu_bar_append_gtk_bookmarks (PanelMenuBar *menubar,
 	}
 
 	for (l = add_files; l; l = l->next) {
-		const char *full_uri;
+		char *full_uri;
+		char *display_uri;
+		char *tooltip;
+		char *basename;
 
 		uri = l->data;
-		full_uri = gnome_vfs_uri_get_path (uri);
+		full_uri = gnome_vfs_uri_to_string (uri,
+						    GNOME_VFS_URI_HIDE_NONE);
 
 		display_uri = gnome_vfs_format_uri_for_display (full_uri);
 		/* Translators: %s is a URI */
@@ -486,6 +508,7 @@ panel_menu_bar_append_gtk_bookmarks (PanelMenuBar *menubar,
 
 		g_free (basename);
 		g_free (tooltip);
+		g_free (full_uri);
 		gnome_vfs_uri_unref (uri);
 	}
 
@@ -764,8 +787,8 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 					     &menubar->priv->desktop_menu);
 
 	menubar->priv->applications_item = gtk_image_menu_item_new_with_label (_("Applications"));
-	image = gtk_image_new_from_stock (PANEL_STOCK_GNOME_LOGO,
-					  panel_menu_bar_icon_get_size ());
+	image = gtk_image_new_from_icon_name (PANEL_GNOME_LOGO_ICON,
+					      panel_menu_bar_icon_get_size ());
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
 				       image);
 
@@ -789,7 +812,7 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->desktop_item);
 
-	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_desktop_menu),
+	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_menus),
 				   menubar);
 }
 
@@ -844,7 +867,7 @@ panel_menu_bar_finalize (GObject *object)
 {
 	PanelMenuBar *menubar = (PanelMenuBar *) object;
 
-	panel_lockdown_notify_remove (G_CALLBACK (panel_menu_bar_recreate_desktop_menu),
+	panel_lockdown_notify_remove (G_CALLBACK (panel_menu_bar_recreate_menus),
 				      menubar);
 
 	gconf_client_remove_dir (panel_gconf_get_client (),
