@@ -31,7 +31,6 @@
 
 #include "applet.h"
 #include "panel-widget.h"
-#include "panel-stock-icons.h"
 #include "panel-util.h"
 #include "panel-profile.h"
 #include "panel-globals.h"
@@ -51,6 +50,25 @@ enum {
 	PROP_DND_ENABLED
 };
 
+typedef enum {
+	FIRST_MENU,
+	APPLICATIONS_MENU,
+#define DEFAULT_MENU      APPLICATIONS_MENU
+	SETTINGS_MENU,
+	LAST_MENU
+} MenuPathRoot;
+
+typedef struct {
+	MenuPathRoot  root_id;
+	char         *scheme;
+	char         *filename;
+} MenuPathRootItem;
+
+static MenuPathRootItem root_items [] = {
+	{ APPLICATIONS_MENU, "applications", "applications.menu" },
+	{ SETTINGS_MENU,     "settings",     "settings.menu"     }
+};
+
 struct _PanelMenuButtonPrivate {
 	PanelToplevel         *toplevel;
 	guint                  gconf_notify;
@@ -62,6 +80,7 @@ struct _PanelMenuButtonPrivate {
 	char                  *custom_icon;
 	char                  *tooltip;
 
+	MenuPathRoot           path_root;
 	guint                  use_menu_path : 1;
 	guint                  use_custom_icon : 1;
 	guint                  dnd_enabled : 1;
@@ -75,6 +94,70 @@ static AtkObject *panel_menu_button_get_accessible  (GtkWidget       *widget);
 
 static GObjectClass *parent_class;
 
+static const char *
+panel_menu_path_root_to_filename (MenuPathRoot path_root)
+{
+	const char *retval;
+	int         i;
+
+	retval = NULL;
+
+	for (i = 0; i < G_N_ELEMENTS (root_items); i++) {
+		if (root_items [i].root_id == path_root) {
+			retval = root_items [i].filename;
+			break;
+		}
+	}
+
+	return retval;
+}
+
+static const char *
+panel_menu_filename_to_scheme (const char *filename)
+{
+	const char *retval;
+	int         i;
+
+	retval = NULL;
+	
+	if (!filename)
+		return retval;
+
+	for (i = 0; i < G_N_ELEMENTS (root_items); i++) {
+		if (root_items [i].filename &&
+		    !strncmp (filename, root_items [i].filename,
+			      strlen (root_items [i].filename))) {
+			retval = root_items [i].scheme;
+			break;
+		}
+	}
+
+	return retval;
+}
+
+static MenuPathRoot
+panel_menu_scheme_to_path_root (const char *scheme)
+{
+	MenuPathRoot retval;
+	int          i;
+
+	retval = LAST_MENU;
+	
+	if (!scheme)
+		return retval;
+
+	for (i = 0; i < G_N_ELEMENTS (root_items); i++) {
+		if (root_items [i].scheme &&
+		    !strncmp (scheme, root_items [i].scheme,
+			      strlen (root_items [i].scheme))) {
+			retval = root_items [i].root_id;
+			break;
+		}
+	}
+
+	return retval;
+}
+
 static void
 panel_menu_button_instance_init (PanelMenuButton      *button,
 				 PanelMenuButtonClass *klass)
@@ -89,10 +172,9 @@ panel_menu_button_instance_init (PanelMenuButton      *button,
 	button->priv->custom_icon = NULL;
 	button->priv->tooltip = NULL;
 
+	button->priv->path_root       = LAST_MENU;
 	button->priv->use_menu_path   = FALSE;
 	button->priv->use_custom_icon = FALSE;
-
-	button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_MAIN_MENU);
 }
 
 static void
@@ -294,10 +376,15 @@ panel_menu_button_create_menu (PanelMenuButton *button)
 
 	panel_widget = panel_toplevel_get_panel_widget (button->priv->toplevel);
 
-	if (button->priv->use_menu_path && button->priv->menu_path)
-		button->priv->menu = create_applications_menu ("applications.menu",
+	if (button->priv->use_menu_path          &&
+	    button->priv->path_root > FIRST_MENU &&
+	    button->priv->path_root < LAST_MENU) {
+		const char *filename;
+
+		filename = panel_menu_path_root_to_filename (button->priv->path_root);
+		button->priv->menu = create_applications_menu (filename,
 							       button->priv->menu_path);
-	else
+	} else
 		button->priv->menu = create_main_menu (panel_widget);
 
 	gtk_menu_attach_to_widget (GTK_MENU (button->priv->menu),
@@ -615,20 +702,24 @@ panel_menu_button_get_icon (PanelMenuButton *button)
 	MenuTreeDirectory *directory;
         char              *retval;
 
-	if (!button->priv->use_menu_path || !button->priv->menu_path)
-		return NULL;
+	retval = NULL;
 
-	if (!panel_menu_button_create_menu (button))
-		return NULL;
+	if (button->priv->use_custom_icon &&
+	    button->priv->custom_icon)
+		retval = g_strdup (button->priv->custom_icon);
 
-	directory = g_object_get_data (G_OBJECT (button->priv->menu),
-				       "panel-menu-tree-directory");
-	if (!directory)
-		return NULL;
+	if (!retval                     &&
+	    button->priv->use_menu_path &&
+	    button->priv->menu_path     &&
+	    panel_menu_button_create_menu (button)) {
+		directory = g_object_get_data (G_OBJECT (button->priv->menu),
+					       "panel-menu-tree-directory");
+		if (directory)
+			retval = g_strdup (menu_tree_directory_get_icon (directory));
+	}
 
-	retval = g_strdup (menu_tree_directory_get_icon (directory));
-                                                                                                             
-	menu_tree_directory_unref (directory);
+	if (!retval)
+		retval = g_strdup (PANEL_MAIN_MENU_ICON);
 
 	return retval;
 }
@@ -636,14 +727,9 @@ panel_menu_button_get_icon (PanelMenuButton *button)
 static void
 panel_menu_button_set_icon (PanelMenuButton *button)
 {
-	char *icon_path = NULL;
-
-	if (button->priv->use_custom_icon)
-		icon_path = g_strdup (button->priv->custom_icon);
-
-	if (!icon_path)
-		icon_path = panel_menu_button_get_icon (button);
+	char *icon_path;
 
+	icon_path = panel_menu_button_get_icon (button);
 	button_widget_set_icon_name (BUTTON_WIDGET (button), icon_path);
 
 	g_free (icon_path);
@@ -679,13 +765,17 @@ panel_menu_button_set_menu_path (PanelMenuButton *button,
 				 const char      *menu_uri)
 {
 	const char *menu_path;
+	char       *scheme;
 
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
 
-	/*
-	 * Bah, lets just ignore the scheme
-	 */
-	menu_path = split_menu_uri (menu_uri, NULL);
+	scheme    = NULL;
+	menu_path = split_menu_uri (menu_uri, &scheme);
+
+	if (!scheme)
+		return;
+
+	button->priv->path_root = panel_menu_scheme_to_path_root (scheme);
 
 	if (!button->priv->menu_path && (!menu_path || !menu_path [0]))
 		return;
@@ -697,8 +787,7 @@ panel_menu_button_set_menu_path (PanelMenuButton *button,
 	g_free (button->priv->menu_path);
 	button->priv->menu_path = NULL;
 
-	if (menu_path && menu_path [0])
-		button->priv->menu_path = g_strdup (menu_path);
+	button->priv->menu_path = g_strdup (menu_path);
 
 	if (button->priv->menu)
 		gtk_menu_detach (GTK_MENU (button->priv->menu));
@@ -825,14 +914,16 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 	g_free (tooltip);
 }
 
-void
+gboolean
 panel_menu_button_create (PanelToplevel *toplevel,
 			  int            position,
+			  const char    *filename,
 			  const char    *menu_path,
 			  gboolean       use_menu_path,
 			  const char    *tooltip)
 {
 	GConfClient *client;
+	const char  *scheme;
 	const char  *key;
 	char        *id;
 
@@ -843,10 +934,17 @@ panel_menu_button_create (PanelToplevel *toplevel,
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "use_menu_path");
 	gconf_client_set_bool (client, key, use_menu_path, NULL);
 
-	if (use_menu_path && menu_path && menu_path [0]) {
-		char *menu_uri;
+	scheme = panel_menu_filename_to_scheme (filename);
 
-		menu_uri = g_strconcat ("applications:", menu_path, NULL);
+	if (filename && !scheme) {
+		g_warning ("Failed to find menu scheme for %s\n", filename);
+		return FALSE;
+	}
+
+	if (use_menu_path && menu_path && menu_path [0] && scheme) {
+		char       *menu_uri;
+
+		menu_uri = g_strconcat (scheme, ":", menu_path, NULL);
 
 		key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "menu_path");
 		gconf_client_set_string (client, key, menu_uri, NULL);
@@ -861,6 +959,8 @@ panel_menu_button_create (PanelToplevel *toplevel,
 
 	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+
+	return TRUE;
 }
 			  
 void
@@ -900,8 +1000,11 @@ panel_menu_button_set_dnd_enabled (PanelMenuButton *button,
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-		gtk_drag_source_set_icon_stock (GTK_WIDGET (button),
-						PANEL_STOCK_MAIN_MENU);
+		/* FIXME: waiting for bug #116577
+		icon = panel_menu_button_get_icon (button);
+		gtk_drag_source_set_icon_name (GTK_WIDGET (button), icon);
+		g_free (icon);
+		*/
 		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
